/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Sensor driver for adxl210
*
* Provides low and high level functions for initializing and sampling of the
* adxl acceleration sensor
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-06-11 (yyyy-mm-dd)
*
* version 001 (2003-12-03, cdecker)
*   - driver on new software architecure
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#include "adxl210_xy.h"
#include "../boards/coreboard.h"
#include "../boards/sensorboard.h"
#include "../system/acl.h"
#include "adxl210.h"

/**
* Initializes the Adxl
* Sets power to output and sampling pins to input.
*
* @return returns 0 if no error
*/
#ifdef PIN_ACCL_ADXL_XOUT
uint8_t AcclSensorInit()
{
	bit_clear(TRIS_ACCL_ADXL_POWER);		// set power pin to output
	bit_set(TRIS_ACCL_ADXL_XOUT);			// set sample pinx to input
	bit_set(TRIS_ACCL_ADXL_YOUT);			// set sample piny to input

 	AcclSensorOff();						// power off for light

	return 0;								// indicate no error
}
#endif


/**
* Setups the microcontroller for sampling
* @return returns 0 if no error
*/
uint8_t AcclSensorPrepare()
{

	//nothing to do here??? yes: wait for adxl to settle after turn on
	return 0;
}





/**
* Gets the duty cycle (raw value) from the accel sensor value
* @param dutyHi duration in the duty cycle where the signal is high
* @param dutyLo duration in the duty cycle where the signal is low
* @return returns 0 if no error
*/
uint8_t AcclXSensorGetDuty(uint16_t *dutyHi, uint16_t *dutyLo) wparam
{
	uint16_t xt1,xt2;


	xt1=0;							// two commands for saving assembler space
	xt2=0;


	while( input(PIN_ACCL_ADXL_XOUT) );				// wait for adx = low
	while( !input(PIN_ACCL_ADXL_XOUT) );				// wait for adx = high
	while( input(PIN_ACCL_ADXL_XOUT))	++xt1;			// count duration
	while( !input(PIN_ACCL_ADXL_XOUT))	++xt2;			// count duration

	*dutyHi = xt1;
	*dutyLo = xt2;

	return 0;
}

/**
* Gets the duty cycle (raw value) from the accel sensor value
* @param dutyHi duration in the duty cycle where the signal is high
* @param dutyLo duration in the duty cycle where the signal is low
* @return returns 0 if no error
*/
uint8_t AcclYSensorGetDuty(uint16_t *dutyHi, uint16_t *dutyLo) wparam
{
	uint16_t yt1,yt2;


	yt1=0;							// two commands for saving assembler space
	yt2=0;


	while( input(PIN_ACCL_ADXL_YOUT) );				// wait for adx = low
	while( !input(PIN_ACCL_ADXL_YOUT) );				// wait for adx = high
	while( input(PIN_ACCL_ADXL_YOUT))	++yt1;			// count duration
	while( !input(PIN_ACCL_ADXL_YOUT))	++yt2;			// count duration

	*dutyHi = yt1;
	*dutyLo = yt2;

	return 0;
}


/**
* Returns the acceleration in g scale
* Unit: for one g (scale=10 means +-1g is +-10 digits)
* @param g the accl value in g
* @return returns 0 if no error
*/
uint8_t AcclXSensorGetGravity(int16_t* g) wparam
{
	uint16_t dutyHi,dutyLo;
	uint8_t ret;

	AcclXSensorGetDuty(&dutyHi, &dutyLo);
	ret = AcclSensorGetGravity(g, dutyHi, dutyLo);

	return ret;
}

/**
* Returns the acceleration in g scale
* Unit: for one g (scale=10 means +-1g is +-10 digits)
* @param g the accl value in g
* @return returns 0 if no error
*/
uint8_t AcclYSensorGetGravity(int16_t* g) wparam
{
	uint16_t dutyHi,dutyLo;
	uint8_t ret;

	AcclYSensorGetDuty(&dutyHi, &dutyLo);
	ret = AcclSensorGetGravity(g, dutyHi, dutyLo);

	return ret;
}


/**
* Packs the accl sensor value for the x axis directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value temperature sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AcclXSensorPackInACL(int16_t value) wparam
{

	if ( ACLGetRemainingPayloadSpace() > 5 )
	{
		ACLAddNewType(ACL_TYPE_SGX_HI,ACL_TYPE_SGX_LO); // SGX
		ACLAddData((uint8_t)(value>>8));					// value
		ACLAddData((uint8_t)(value&0x00FF));				// value
		ACLAddData(0);									// index = 0
	}
	else
	{
		return 1; 	// not enough space left
	}

	return 0; // no error
}


/**
* Packs the accl sensor value for the y axis directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value temperature sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AcclYSensorPackInACL(int16_t value) wparam
{

	if ( ACLGetRemainingPayloadSpace() > 5 )
	{
		ACLAddNewType(ACL_TYPE_SGY_HI,ACL_TYPE_SGY_LO); // SGY
		ACLAddData((uint8_t)(value>>8));					// value
		ACLAddData((uint8_t)(value&0x00FF));				// value
		ACLAddData(0);									// index = 0
	}
	else
	{
		return 1; 	// not enough space left
	}

	return 0; // no error
}

