/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Sensor driver for microphone SP0101NC1
*
* Provides low and high level functions for initializing and sampling of the
* microphone
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-06-11 (yyyy-mm-dd)
*
* version 001 (2003-12-03, cdecker)
*   - driver on new software architecure
*
* version 002 (2003-12-04, cdecker)
*   - error in volume computation which causes infinte loop corrected
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#include "sp0101nc1.h"
#include "../boards/coreboard.h"
#include "../boards/sensorboard.h"
#include "../system/acl.h"

#ifndef PIN_AUDIO_MIC_OUT
#error No pin for audio mic declared
#endif

/**
* Initializes the Audio microphone amp
* Sets power to output and sampling pin to input.
*
* @return returns 0 if no error
*/
#ifdef PIN_AUDIO_MIC_OUT
uint8_t AudioSensorInit()
{
	bit_clear(TRIS_AUDIO_MIC_POWER);		// set power pin to output
	bit_set(TRIS_AUDIO_MIC_OUT);			// set sample pin to input

	AudioSensorOff();						// power off for light
	return 0;								// indicate no error
}
#endif



/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t AudioSensorGet(uint8_t* value) wparam
{

	*value = ADCRead8();
	return 0;
}



/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t AudioSensorPrepare()
{
	// set channel
	if ( !ADCIsChSet(ADCH_AUDIO_MIC) ) {
		ADCSetCh(ADCH_AUDIO_MIC);
	}

	// set conversion clock
	ADCSetClock(0);

	// start ADC
	ADCEnable();

	return 0;
}



/**
* Computes the average power of the audio signal given an
* array of sampled values
* @param AudioRawData array of sampled audio values
* @param AudioArraySize array size
* @return the volume
*/
uint8_t AudioCalcVolume(uint8_t* AudioRawData, uint8_t AudioArraySize) wparam
{
	uint8_t i;
	uint8_t DCoffset;
	uint8_t Pave;
	uint16_t TempSum;
	uint8_t temp;



	//first calc DCoffset as median = E{AudioRawData}

	TempSum=0;
	for (i=0;i<AudioArraySize;i++)
	{
		TempSum=TempSum+ (uint16_t) AudioRawData[i];
	}
	DCoffset=(uint8_t) (TempSum/AudioArraySize);


	//now calc average Power of Signal

	Pave=0;
	temp=0;
	TempSum=0;
	for (i=0;i<AudioArraySize;i++)
	{
		if (AudioRawData[i] <= DCoffset)
			{
				temp= DCoffset - AudioRawData[i];
			}
		else
			{
				temp= AudioRawData[i]-DCoffset;
			}

		TempSum=TempSum+(uint16_t) temp	;
	}

	Pave=(uint8_t) (TempSum/AudioArraySize);


	return (Pave);
}



/**
* Computes the average power of the audio signal.
* This power is computed
* @param volume pointer for storing the volume level
* @return returns 0 if no error
*/
uint8_t AudioSensorGetVolume(uint8_t* volume) wparam
{
	uint16_t TempSum;
	uint16_t TempSum2;

	uint8_t i;
	uint8_t DCoffset;
	uint8_t Pave;
	uint8_t temp;


	//first calc DCoffset as median = E{AudioRawData}
	// 2 loops acquire 512 samples, this method avoids data types longer than a uint16_t

	TempSum=0;
	for (i=0;i<128;i++)
	{
		AudioSensorGet(&temp);
		TempSum=TempSum+temp;
	}

/*
	TempSum2=0;
	for (i=0;i<256;i++)
	{
		AudioSensorGet(&temp);
		TempSum2=TempSum2+temp;
	}


	DCoffset=(uint8_t) ( (TempSum>>9) + (TempSum2>>9) );
*/
	DCoffset=(uint8_t) ( (TempSum>>7) );



	//now calc average Power of Signal
	// same procedure as for DCoffset

	TempSum=0;
	TempSum2=0;

	for (i=0;i<128;i++)
	{
		AudioSensorGet(&temp);
		if (temp <= DCoffset)
		{
			temp= DCoffset - temp;
		}
		else
		{
			temp= temp -DCoffset;
		}
		TempSum=TempSum+temp;
	}

/*
	for (i=0;i<256;i++)
	{
		AudioSensorGet(&temp);
		if (temp <= DCoffset)
		{
			temp= DCoffset - temp;
		}
		else
		{
			temp= temp -DCoffset;
		}
		TempSum=TempSum+temp;
	}

	Pave= (uint8_t) ( (TempSum>>9) + (TempSum2>>9) );
*/
	Pave= (uint8_t) ( (TempSum>>7) );

	*volume=Pave;
	return 0;

}

/**
* Packs the audio sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param type type of audio sensor value (e.g. 1 for mean volume)
* @param data pointer to data array of audio value(s)
* @param dataLength length of data array
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AudioSensorPackInACL(uint8_t type, uint8_t* __PIC_data, uint8_t dataLength) wparam
{
	uint8_t i;

	if ( ACLGetRemainingPayloadSpace() > 5 + dataLength)
	{
		ACLAddNewType(ACL_TYPE_SAU_HI,ACL_TYPE_SAU_LO); // SAU
		ACLAddData(type);								// value
		for (i = 0; i < dataLength; i++)
		{
			ACLAddData(__PIC_data[i]);						// value(s)
		}
		ACLAddData(0);									// index = 0
	}
	else
	{
		return 1; 	// not enough space left
	}

	return 0; // no error
}

