/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Sensor driver for tc74 il2c temp sensor
*
* Provides low level functions for reading the tc74 temperature sensor
* via I2C.
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-07-06 (yyyy-mm-dd)
*
* modified by cdecker@teco.edu
*
* version 001:
*   - adapt driver to new i2c wrapper
*
* version 002:
*   - ACL pack function
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#include "tc74.h"
#include "../boards/coreboard.h"
#include "../boards/sensorboard.h"
#include "../system/acl.h"
#include "../boards/sensori2c.h"

/**
* Initializes the temperature sensor
* Uses the Sensor I2C Bus
*
* @return returns 0 if no error
*/

uint8_t TemperatureSensorInit()
{
#ifdef PIN_TEMP_TC74_POWER
	SensorI2CInit();						// needs i2c

	bit_clear(TRIS_TEMP_TC74_POWER);
	TemperatureSensorOff();

	return 0;								// indicate no error
#endif
}


/**
* Setups the I2C bus for the temperature sensor
* @return returns 0 if no error
*/
uint8_t TemperatureSensorPrepare()
{
	SensorI2COn();
	return 0;
}


/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t TemperatureSensorGet(int8_t* value) wparam
{

	int8_t __PIC_data;



   	SensorI2CStart();

	if (SensorI2CWrite(I2C_TEMPERATURE_ID)) {SensorI2CStop();return 2;}		//error: no ack, write to device
	if (SensorI2CWrite(0)) {SensorI2CStop();return 3;}				//error: no ack, read from temp reg

	SensorI2CStart();
	if (SensorI2CWrite(I2C_TEMPERATURE_ID+1)) {SensorI2CStop();return 4;}		//error: no ack, read from device
	__PIC_data=SensorI2CRead(0);

	SensorI2CStop();

	*value=(__PIC_data-3); // actually data-3.4 degress celsius

	return 0;
}




/**
* Packs the temperature sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value temperature sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t TemperatureSensorPackInACL(int8_t value) wparam
{

	if ( ACLGetRemainingPayloadSpace() > 4 )
	{
		ACLAddNewType(ACL_TYPE_STE_HI,ACL_TYPE_STE_LO); // STE
		ACLAddData(value);								// value
		ACLAddData(0);									// index = 0
	}
	else
	{
		return 1; 	// not enough space left
	}

	return 0; // no error
}

