/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Sensor driver for light sensor tsl25500
*
* Provides low and high level functions for initializing and sampling of the
* TSL2550 light sensor.
*
* Author: Christian Decker (cdecker@teco.edu)
*
* version 001 (2003-12-03, cdecker)
*   - driver on new software architecure
* version 002 (2004-02-05, sabin (wendhack@teco.edu))
* 	- pin corrected: PIN_LIGHT_TSL2550_SDA
* version 003 (2004-04-08, sabin (wendhack@teco.edu))
*	- gives uint16_t instead of uint8_t to AmbientLightIRSensorGet().
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#include "tsl2550.h"
#include "../boards/coreboard.h"
#include "../boards/sensorboard.h"
#include "../system/acl.h"
#include "../boards/sensori2c.h"
#include "tsl250_generic.h"

#ifndef PIN_LIGHT_TSL2550_SDA
#error No pins for light sensor found
#endif

/**
* Switches the light sensor on
*/
#define AmbientLightSensorOn()	output_high(PIN_LIGHT_TSL2550_POWER)

/**
* Switches the light sensor off
*/
#define AmbientLightSensorOff()	output_low(PIN_LIGHT_TSL2550_POWER)


/**
* Initializes the TSL2550 light sensor.
* Sets power to output and sampling pin to input.
*
* @return returns 0 if no error
*/
#ifdef PIN_LIGHT_TSL2550_SDA
uint8_t AmbientLightSensorInit()
{
	SensorI2CInit();							// needs I2C

	bit_clear(TRIS_LIGHT_TSL2550_POWER);		// set power pin to output
	AmbientLightSensorOff();							// power off for light

	return 0;								// indicate no error
}
#endif


/**
* Supports the SensorGet functions in respect to the calculation
* of the light level for both the visible light and infrared light
* @param data sensor value from ambient light sensor
* @return light level from sensor value
*/
uint16_t AmbientLightCalcSensorValue(uint8_t __PIC_data) wparam
{
	uint8_t c, sv, sn;
	uint16_t ldata, cv;

	//Calculate C
	c=__PIC_data;
	c&=0x70;
	c= c >> 4;
	//Calculate SV
	sv=0;
	bit_set(sv, c);
	//Calculate SN
	sn=__PIC_data;
	sn&=0x0F;
	//Calculate CV
	cv=16*((uint16_t)sv-1);
	//Calculate ldata
	ldata=cv+((uint16_t)sv*(uint16_t)sn);

	return ldata;

}

/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t AmbientLightVisibleSensorGet(uint16_t* value) wparam
{

	uint8_t __PIC_data;

	SensorI2CStart();
		if (SensorI2CWrite(0x72)) {SensorI2CStop();return 1;}		//error: no ack, write to device
		if (SensorI2CWrite(0x43)) {SensorI2CStop();return 2;}		//error: no ack, read from temp reg
	SensorI2CStop();

	SensorI2CStart();
		if (SensorI2CWrite(0x73)) {SensorI2CStop();return 3;}		//error: no ack, read from device
		__PIC_data=SensorI2CRead(0);
	SensorI2CStop();

	*value=AmbientLightCalcSensorValue(__PIC_data);

	return 0;
}

/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t AmbientLightIRSensorGet(uint16_t* value) wparam
{

	uint8_t __PIC_data, sn;

	SensorI2CStart();
		if (SensorI2CWrite(0x72)) {SensorI2CStop();return 1;}		//error: no ack, write to device
		if (SensorI2CWrite(0x83)) {SensorI2CStop();return 2;}		//error: no ack, read from temp reg
	SensorI2CStop();

	SensorI2CStart();
		if (SensorI2CWrite(0x73)) {SensorI2CStop();return 3;}		//error: no ack, read from device
		__PIC_data=SensorI2CRead(0);
	SensorI2CStop();


	*value=AmbientLightCalcSensorValue(__PIC_data);

	return 0;
}

/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t AmbientLightSensorPrepare()
{
	SensorI2COn();
	return 0;
}


/**
* Packs the light sensor value for visible light directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value force sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AmbientLightVisibleSensorPackInACL(uint16_t value) wparam
{

	uint8_t tmp[2];
	uint8_t ret;

	tmp[0] = (uint8_t)(value>>8);
	tmp[1] = (uint8_t)(value&0x00FF);


	ret = LightGenericSensorPackInACL(22, tmp, 2, 0);  // 22 = TSL2550, Visible

	return ret;
}

/**
* Packs the light sensor value for visible light directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value force sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AmbientLightIRSensorPackInACL(uint16_t value) wparam
{
	uint8_t tmp[2];
	uint8_t ret;

	tmp[0] = (uint8_t)(value>>8);
	tmp[1] = (uint8_t)(value&0x00FF);


	ret = LightGenericSensorPackInACL(23, tmp, 2, 0);  // 23 = TSL2550, IR

	return ret;
}


