/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Sensor driver for measuring the battery voltage
*
* This is a particle core board driver, i.e. the voltage measurement is done
* on the core board and not on a sensor board.
* Provides low and high level functions for initializing and sampling of the
* voltage measurment.
*
* Author: Christian Decker (cdecker@teco.edu)
* Date: 2003-12-11 (yyyy-mm-dd)
*
* version 001 (2003-12-11, cdecker)
*   - init version
*
* version 002 (2004-04-05) by Klaus Schmidt (schmidtk@teco.edu)
*	- SPart & Particle version
*
* version 003 (2004-11-03) by Michael Demel	(demel@teco.edu)
*   - Voltage Sensor adjust to 3,3V (VRef)
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/

#include "voltage.h"
#include "../boards/coreboard.h"
#include "../boards/sensorboard.h"
#include "../system/acl.h"


/**
* On/Off functions are not need, because this feature is not supported
*/

/**
* Initializes the Voltage sensor.
* @return returns 0 if no error
*/
#ifdef PIN_VOLTAGE
uint8_t VoltageSensorInit()
{
	// nothing to be done here, since PCInit() function
	return 0;
}
#else
	#error "No Voltage sensor declared"
#endif


/**
* Packs the voltage directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value voltage measurement [mV]
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t VoltageSensorPackInACL(uint16_t value) wparam
{

	if ( ACLGetRemainingPayloadSpace() > 4 )
	{
		ACLAddNewType(ACL_TYPE_SVC_HI,ACL_TYPE_SVC_LO); // SVC
		ACLAddData((uint8_t)(value>>8));					// value
		ACLAddData((uint8_t)(value&0x00FF));				// value
	}
	else
	{
		return 1; 	// not enough space left
	}

	return 0; // no error
}


#ifndef __SPart202_C__		// normal voltage-driver for coreboard
/**
* Gets the raw sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t SampledVoltageSensorGet(uint16_t* value) wparam
{

	*value = ADCRead10();
	return 0;
}

/**
* Gets the voltage in millivolt (mV)
* @param value voltage in millivolt (mV)
* @return returns 0 if no error
*/
uint8_t VoltageSensorGet(uint16_t* value) wparam
{
	uint16_t v,v2;
	uint8_t ret;

	ret = SampledVoltageSensorGet(&v);

	// 3300 mV are sampled as 1024
	// Formula v2 = (v * 3) / 4  returns 3328
	// Error: 0.8%

	v2 = v<<1;
	v2 += v;
	v2 += (v>>2);

	*value = v2;

	return ret;
}

/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t VoltageSensorPrepare()
{
	// set channel
	if ( !ADCIsChSet(ADCH_VOLTAGE) ) {
		ADCSetCh(ADCH_VOLTAGE);
	}

	// set conversion clock
	ADCSetClock(0);

	// start ADC
	ADCEnable();

	return 0;
}



#else



/**
* Setups the microcontroller of the Coreboard for sampling
* Sets the correct AD channel, if neccessary
* @result returns 0 if no error
*/
uint8_t VoltageSensorPrepare() {

	uint8_t result;

	result = RPC(20000, RPC_VoltageSensorPrepare,0 ,0 ,0);

	return result;
}

/**
* Gets the value of the voltage sensor in mV over RPC, saves
* it to get it with RPC_ReturnPointerResult
* @*value : the voltage value as a uint16_t pointer
* @result : returns 0 if no error
*/
uint8_t VoltageSensorGet(uint16_t* value) wparam
{
	uint8_t result;
	uint16_t temp;

	result = RPC(20000, RPC_VoltageSensorGet, 0, 0, 0);

	temp = ( 0x00 | RPC(20000,RPC_ReturnPointerResult, 1, 0, 0) );
	*value = (0x00 | (RPC(20000, RPC_ReturnPointerResult, 0, 0, 0) << 8) );

	*value = ( *value | temp );

	return result;

}
#endif			// __SPart202_C__
